# -*- coding: utf-8 -*-

"""
infermedica_api.webservice
~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains classes and function responsible for making API requests.
"""

import json
import platform

import requests
from . import __version__, exceptions, models, API_CONFIG, DEFAULT_API_VERSION

if platform.python_version_tuple()[0] == '3':
    basestring = (str,bytes)


class SearchFilters(object):
    """Simple class to hold search filter constants."""
    SYMPTOMS = "symptom"
    RISK_FACTORS = "risk_factor"
    LAB_TESTS = "lab_test"

    ALL = [SYMPTOMS, RISK_FACTORS, LAB_TESTS]

SEARCH_FILTERS = SearchFilters()


class API(object):
    """Class which handles requests to the Infermedica API."""

    # User-Agent for HTTP request
    library_details = "requests %s; python %s" % (requests.__version__, platform.python_version())
    user_agent = "Infermedica-API-Python %s (%s)" % (__version__, library_details)

    def __init__(self, **kwargs):
        """
        Initialize API object.

        Usage::
            >>> import infermedica_api
            >>> api = infermedica_api.API(app_id='YOUR_APP_ID', app_key='YOUR_APP_KEY')
        """
        self.api_version = kwargs.get("api_version", DEFAULT_API_VERSION)
        self.app_id = kwargs["app_id"]  # Mandatory parameter, so not using `dict.get`
        self.app_key = kwargs["app_key"]  # Mandatory parameter, so not using `dict.get`
        self.default_headers = self.__calculate_headers(kwargs)

        if self.api_version in kwargs.get("api_definitions", {}) or {}:
            self.endpoint = kwargs["api_definitions"][self.api_version]['endpoint']
            self.api_methods = kwargs["api_definitions"][self.api_version]['methods']
        else:
            self.endpoint = API_CONFIG[self.api_version]['endpoint']
            self.api_methods = API_CONFIG[self.api_version]['methods']

    def __calculate_headers(self, parameters):
        headers = parameters.get("default_headers", {})
        if parameters.get("model", None):
            headers.update({
                "Model": parameters["model"]
            })

        if parameters.get("dev_mode", None) and parameters["dev_mode"] == True:
            headers.update({
                "Dev-Mode": "true"
            })

        return headers

    def __get_url(self, method):
        return self.endpoint + self.api_version + method

    def __get_headers(self, override):
        """Returns default HTTP headers."""
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": self.user_agent,
            "app_id": self.app_id,
            "app_key": self.app_key
        }
        headers.update(self.default_headers)
        headers.update(override)
        return headers

    def __api_call(self, url, method, **kwargs):

        kwargs['headers'] = self.__get_headers(kwargs['headers'] or {})

        response = requests.request(method, url, **kwargs)

        return self.__handle_response(response)

    def __handle_response(self, response):
        """
        Validates HTTP response, if response is correct decode json data and returns dict object.
        If response is not correct raise appropriate exception.

        :returns: dict or list with response data
        :rtype: dict or list
        :raises:
            infermedica_api.exceptions.BadRequest,
            infermedica_api.exceptions.UnauthorizedAccess,
            infermedica_api.exceptions.ForbiddenAccess,
            infermedica_api.exceptions.ResourceNotFound,
            infermedica_api.exceptions.MethodNotAllowed,
            infermedica_api.exceptions.ServerError,
            infermedica_api.exceptions.ConnectionError
        """
        status = response.status_code
        content = response.content.decode('utf-8')

        if 200 <= status <= 299:
            return json.loads(content) if content else {}
        elif status == 400:
            raise exceptions.BadRequest(response, content)
        elif status == 401:
            raise exceptions.UnauthorizedAccess(response, content)
        elif status == 403:
            raise exceptions.ForbiddenAccess(response, content)
        elif status == 404:
            raise exceptions.ResourceNotFound(response, content)
        elif status == 405:
            raise exceptions.MethodNotAllowed(response, content)
        elif 500 <= status <= 599:
            raise exceptions.ServerError(response, content)
        else:
            raise exceptions.ConnectionError(response, content)

    def __get(self, method, params=None, headers=None):
        """Wrapper for a GET API call."""
        return self.__api_call(self.__get_url(method), "GET", headers=headers, params=params)

    def __post(self, method, data, headers=None):
        """Wrapper for a GET API call."""
        return self.__api_call(self.__get_url(method), "POST", headers=headers, data=data)

    def info(self):
        """Makes an API request and returns basic API model information."""
        try:
            return self.__get(self.api_methods['info'])
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'info')

    def search(self, phrase, sex=None, max_results=8, filters=None):
        """
        Makes an API search request and returns list of dicts containing keys: 'id', 'label' and 'type'.
        Each dict represent an evidence (symptom, lab test or risk factor).
        By default only symptoms are returned, to include other evidence types use filters.

        :param phrase: Phrase to look for.
        :type phrase: str

        :param sex: Sex of the patient 'female' or 'male'.
        :type sex: str

        :param max_results: Maximum number of result to return, default is 8.
        :type max_results: int

        :param filters: List of search filters, taken from SEARCH_FILTERS variable.
        :type filters: list

        :returns: A List of dicts with 'id' and 'label' keys.
        :rtype: list
        """
        try:
            params = {
                'phrase': phrase,
                'max_results': max_results
            }

            if sex:
                params['sex'] = sex

            if filters:
                if isinstance(filters, (list, tuple)):
                    params['type'] = filters
                elif isinstance(filters, basestring):
                    params['type'] = [filters]

                for filter in params['type']:
                    if filter not in SEARCH_FILTERS.ALL:
                        raise exceptions.InvalidSearchFilter(filter)

            return self.__get(self.api_methods['search'], params=params)
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'search')

    def lookup(self, phrase, sex=None):
        """
        Makes an API lookup request and returns evidence details object.

        :param phrase: Phrase to look for.
        :type phrase: str

        :param sex: Sex of the patient 'female' or 'male'.
        :type sex: str

        :returns: Dictionary with details.
        :rtype: dict
        """
        try:
            params = {
                'phrase': phrase
            }
            if sex:
                params['sex'] = sex
            return self.__get(self.api_methods['lookup'], params=params)
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'lookup')

    def diagnosis(self, diagnosis_request, case_id=None):
        """
        Makes an diagnosis API request with provided diagnosis data
        and returns diagnosis question with possible conditions.

        :param diagnosis_request: Diagnosis request object or json request for diagnosis method.
        :type diagnosis_request: :class:`infermedica_api.models.Diagnosis` or dict

        :param case_id: Unique case id for diagnosis
        :type case_id: str

        :returns: A Diagnosis object with api response
        :rtype: :class:`infermedica_api.models.Diagnosis`
        """
        try:
            method = self.api_methods['diagnosis']
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'diagnosis')

        headers = {}
        if case_id:
            headers['Case-Id'] = case_id

        if isinstance(diagnosis_request, models.Diagnosis):
            if not case_id and diagnosis_request.case_id:
                headers['Case-Id'] = diagnosis_request.case_id

            response = self.__post(method, json.dumps(diagnosis_request.get_api_request()), headers=headers)
            diagnosis_request.update_from_api(response)

            return diagnosis_request

        return self.__post(method, json.dumps(diagnosis_request), headers=headers)

    def explain(self, diagnosis_request, target_id):
        """
        Makes an explain API request with provided diagnosis data and target condition.
        Returns explain results with supporting and conflicting evidences.

        :param diagnosis_request: Diagnosis request object or json request for diagnosis method.
        :type diagnosis_request: :class:`infermedica_api.models.Diagnosis` or dict

        :param target_id: Condition id for which explain shall be calculated.
        :type target_id: str

        :returns: A Diagnosis object with api response
        :rtype: :class:`infermedica_api.models.Diagnosis`
        """
        try:
            if isinstance(diagnosis_request, models.Diagnosis):
                request = diagnosis_request.get_explain_request(target_id)
            else:
                request = dict(diagnosis_request, **{'target': target_id})

            response = self.__post(self.api_methods['explain'], json.dumps(request))

            return models.ExplainResults.from_json(response)
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'explain')

    def observation_details(self, _id):
        """
        Makes an API request and returns observation details object.

        :param _id: Observation id
        :type _id: str

        :returns: A Observation object
        :rtype: :class:`infermedica_api.models.Observation`
        """
        try:
            url = self.api_methods['observation_details'].format(**{'id': _id})
            response = self.__get(url)
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'observation_details')

        return models.Observation.from_json(response)

    def observations_list(self):
        """
        Makes an API request and returns list of observation details objects.

        :returns: A ObservationList list object with Observation objects
        :rtype: :class:`infermedica_api.models.ObservationList`
        """
        try:
            response = self.__get(self.api_methods['observations'])
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'observations_list')

        return models.ObservationList.from_json(response)

    def condition_details(self, _id):
        """
        Makes an API request and returns condition details object.

        :param _id: Condition id
        :type _id: str

        :returns:A Condition object
        :rtype: :class:`infermedica_api.models.Condition`
        """
        try:
            url = self.api_methods['condition_details'].format(**{'id': _id})
            response = self.__get(url)
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'condition_details')

        return models.Condition.from_json(response)

    def conditions_list(self):
        """
        Makes an API request and returns list of condition details objects.

        :returns: A ConditionList list object with Condition objects
        :rtype: :class:`infermedica_api.models.ConditionList`
        """
        try:
            response = self.__get(self.api_methods['conditions'])
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'conditions_list')

        return models.ConditionList.from_json(response)

    def symptom_details(self, _id):
        """
        Makes an API request and returns symptom details object.

        :param _id: Symptom id
        :type _id: str

        :returns: A Symptom object
        :rtype: :class:`infermedica_api.models.Symptom`
        """
        try:
            url = self.api_methods['symptom_details'].format(**{'id': _id})
            response = self.__get(url)
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'symptom_details')

        return models.Symptom.from_json(response)

    def symptoms_list(self):
        """
        Makes an API request and returns list of symptom details objects.

        :returns: A SymptomList list object with Symptom objects
        :rtype: :class:`infermedica_api.models.SymptomList`
        """
        try:
            response = self.__get(self.api_methods['symptoms'])
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'symptoms_list')

        return models.SymptomList.from_json(response)

    def lab_test_details(self, _id):
        """
        Makes an API request and returns lab_test details object.

        :param _id: LabTest id
        :type _id: str

        :returns: A LabTest object
        :rtype: :class:`infermedica_api.models.LabTest`
        """
        try:
            url = self.api_methods['lab_test_details'].format(**{'id': _id})
            response = self.__get(url)
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'lab_test_details')

        return models.LabTest.from_json(response)

    def lab_tests_list(self):
        """
        Makes an API request and returns list of lab_test details objects.

        :returns: A LabTestList list object with LabTest objects
        :rtype: :class:`infermedica_api.models.LabTestList`
        """
        try:
            response = self.__get(self.api_methods['lab_tests'])
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'lab_tests_list')

        return models.LabTestList.from_json(response)

    def risk_factor_details(self, _id):
        """
        Makes an API request and returns risk factor details object.

        :param _id: risk factor id
        :type _id: str

        :returns: A RiskFactor object
        :rtype: :class:`infermedica_api.models.RiskFactor`
        """
        try:
            url = self.api_methods['risk_factor_details'].format(**{'id': _id})
            response = self.__get(url)
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'risk_factor_details')

        return models.RiskFactor.from_json(response)

    def risk_factors_list(self):
        """
        Makes an API request and returns list of risk factors details objects.

        :returns: A RiskFactorList list object with RiskFactor objects
        :rtype: :class:`infermedica_api.models.RiskFactorList`
        """
        try:
            response = self.__get(self.api_methods['risk_factors'])
        except KeyError as e:
            raise exceptions.MethodNotAvailableInAPIVersion(self.api_version, 'risk_factors_list')

        return models.RiskFactorList.from_json(response)


__api__ = None
__api_aliased__ = {}


def get_api(alias=None):
    """
    Returns global API object and if present,
    otherwise raise MissingConfiguration exception.

    :param alias: Alias of the API to retrieve
    :type alias: str

    :returns: An API object
    :rtype: :class:`infermedica_api.webservice.API`
    :raises: :class:`infermedica_api.exceptions.MissingConfiguration`
    """
    global __api__
    global __api_aliased__

    if isinstance(alias, basestring):
        try:
            return __api_aliased__[alias]
        except KeyError:
            raise exceptions.MissingConfiguration(alias)

    if __api__ is None:
        raise exceptions.MissingConfiguration()

    return __api__


def configure(options=None, **config):
    """
    Configure and create new global API object with given configuration.
    Configuration can be passed as a dict or separate arguments.
    Returns newly created object.

    Usage:
        >>> import infermedica_api
        >>> infermedica_api.configure({'app_id': 'YOUR_APP_ID', 'app_key': 'YOUR_APP_KEY'})

    ... or:
        >>> import infermedica_api
        >>> infermedica_api.configure(app_id='YOUR_APP_ID', app_key='YOUR_APP_KEY')

    :param options: Dict with configuration data
    :type options: dict

    :returns: An API object
    :rtype: :class:`infermedica_api.webservice.API`
    """
    global __api__
    global __api_aliased__

    configuration = dict(options or {}, **config)

    if 'alias' in configuration and isinstance(configuration['alias'], basestring):
        __api_aliased__[configuration['alias']] = API(**configuration)

        if configuration.get('default', False):
            __api__ = __api_aliased__[configuration['alias']]

        return __api_aliased__[configuration['alias']]

    __api__ = API(**configuration)

    return __api__



# -*- coding: utf-8 -*-

"""
infermedica_api.models.symptom
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains symptom models for data returned from api.
"""
from .base import BaseModel, BaseModelList


class Symptom(BaseModel):
    """Model class for API symptoms details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs Symptom object from given dict and returns it.

        :param json: Dict with symptom values
        :type json: dict

        :returns: Symptom details object
        :rtype: :class:`infermedica_api.models.Symptom`
        """
        return Symptom(**json)


class SymptomList(BaseModelList):
    """Model class for API list of symptom details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs SymptomList object from list of dicts and returns it.

        :param json: List with symptom details dict values
        :type json: list

        :returns: Symptoms details list object
        :rtype: :class:`infermedica_api.models.SymptomList`
        """
        mapping = {}
        for i, item in enumerate(json):
            item = Symptom(**item)
            mapping[item.id] = i
        return SymptomList(json, mapping=mapping)

    def get_symptom_details(self, _id):
        return self._get_details(_id)



# -*- coding: utf-8 -*-

"""
infermedica_api.exceptions
~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains the set of API request exceptions.
"""


class ConnectionError(Exception):
    """API connection error."""

    def __init__(self, response, content=None):
        self.response = response
        self.content = content

    def __str__(self):
        message = "Failed."
        if hasattr(self.response, 'status_code'):
            message += " Response status: %s." % (self.response.status_code)
        if hasattr(self.response, 'reason'):
            message += " Reason: %s." % (self.response.reason)
        if self.content is not None:
            message += " Error message: " + str(self.content)
        return message


class BadRequest(ConnectionError):
    """400 Bad Request."""


class UnauthorizedAccess(ConnectionError):
    """401 Unauthorized."""


class ForbiddenAccess(ConnectionError):
    """403 Forbidden."""


class ResourceNotFound(ConnectionError):
    """404 Not found."""


class ServerError(ConnectionError):
    """5xx Server Error."""


class MissingConfiguration(Exception):
    """API not configured."""

    def __init__(self, alias=None):
        self.alias = alias

    def __str__(self):
        if self.alias:
            return "API credentials for alias '%s' has not been configured." % self.alias
        return "API credentials has not been configured."


class MethodNotAllowed(ConnectionError):
    """405 Method Not Allowed."""


class MethodNotAvailableInAPIVersion(Exception):
    """Try to call API method, which is not available."""

    def __init__(self, current_api_version, method):
        self.api_version = current_api_version
        self.method = method

    def __str__(self):
        return "Method '%s' is not available in the %s api version." % (self.method, self.api_version)


class InvalidSearchFilter(Exception):
    """API not configured."""

    def __init__(self, filter):
        self.filter = filter

    def __str__(self):
        return "Invalid search filter: '%s'." % self.filter



# -*- coding: utf-8 -*-

"""
infermedica_api.models.diagnosis
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains models for data returned from api as well as object to construct api requests,
related to /diagnosis method.
"""
import warnings

from .base import BaseModel, BaseModelList, ModelCommon


class ConditionResult(BaseModel):
    """Model class for API condition results objects from diagnosis calls."""

    @staticmethod
    def from_json(json):
        """
        Constructs ConditionResult object from given dict and returns it.

        :param json: Dict with condition result values
        :type json: dict

        :returns: Condition result details object
        :rtype: :class:`infermedica_api.models.ConditionResult`
        """
        return ConditionResult(**json)


class ConditionResultList(BaseModelList):
    """Model class for API list of condition result objects from diagnosis calls."""

    @staticmethod
    def from_json(json):
        """
        Constructs ConditionResultList object from list of dicts and returns it.

        :param json: List with condition result details dict values
        :type json: list

        :returns: Condition result details list object
        :rtype: :class:`infermedica_api.models.ConditionResultList`
        """
        mapping = {}
        for i, item in enumerate(json):
            item = ConditionResult(**item)
            mapping[item.id] = i
        return ConditionResultList(json, mapping=mapping)

    def get_condition_details(self, _id):
        return self._get_details(_id)


class DiagnosisQuestion(BaseModel):
    """Model class for API question objects from diagnosis calls."""

    @staticmethod
    def from_json(json):
        """
        Constructs DiagnosisQuestion object from given dict and returns it.

        :param json: Dict with diagnosis question values
        :type json: dict

        :returns: Diagnosis question details object
        :rtype: :class:`infermedica_api.models.DiagnosisQuestion`
        """
        return DiagnosisQuestion(**json)


class Diagnosis(ModelCommon):
    """
    Model class for handling diagnosis requests and responses.
    It construct diagnosis request and is updated after next diagnosis calls.
    It will contain diagnosis questions, as well as results.
    """

    def __init__(self, sex, age, time=None, case_id=None):
        """
        Initialize diagnosis object with basic information about patient
        and optionally with examination time.

        :param sex: Patient's sex ("female" or "male")
        :type sex: str
        :param age: Patient's age
        :type age: int
        :param time: (optional) Time of diagnosis evaluation (ISO8601 formatted)
        :type time: str
        :param case_id: Unique case id for diagnosis
        :type case_id: str
        """
        self.patient_sex = sex
        self.patient_age = age
        self.evaluation_time = time

        self.symptoms = []
        self.lab_tests = []
        self.risk_factors = []

        self.pursued = []

        self.question = None
        self.conditions = ConditionResultList()
        self.extras = {}
        self.extras_permanent = {}

        self.case_id = case_id

    @property
    def observations(self):
        return self.symptoms

    def __add_evidence(self, collection, _id, state, time):
        """Helper function to update evidence list."""
        evidence = {
            "id": _id,
            "choice_id": state
        }

        if time:
            evidence['observed_at'] = time

        collection.append(evidence)

    def add_observation(self, _id, state, time=None):
        """
        Adds observation with given presence to evidence list.

        :param _id: Observation id
        :type _id: str
        :param state: Observation presence, one of three values ("present", "absent" or "unknown")
        :type state: str
        :param time: (optional) Observation occurrence time (ISO8601 formatted)
        :type time: str
        """
        warnings.warn("Function add_observation is deprecated, please use add_symptom.",
                      category=DeprecationWarning)
        self.add_symptom(_id, state, time)

    def add_symptom(self, _id, state, time=None):
        """
        Adds symptom with given presence to evidence list.

        :param _id: Symptom id
        :type _id: str
        :param state: Symptom presence, one of three values ("present", "absent" or "unknown")
        :type state: str
        :param time: (optional) Symptom occurrence time (ISO8601 formatted)
        :type time: str
        """
        self.__add_evidence(self.symptoms, _id, state, time)

    def add_lab_test(self, _id, state, time=None):
        """
        Adds laboratory test with given presence to evidence list.

        :param _id: Laboratory test id
        :type _id: str
        :param state: Laboratory test presence, one of three values ("present", "absent" or "unknown")
        :type state: str
        :param time: (optional) Laboratory test occurrence time (ISO8601 formatted)
        :type time: str
        """
        self.__add_evidence(self.lab_tests, _id, state, time)

    def add_risk_factor(self, _id, state, time=None):
        """
        Adds risk factor with given presence to evidence list.

        :param _id: Risk factor id
        :type _id: str
        :param state: Risk factor presence,
        one of three values ("present", "absent" or "unknown")
        :type state: str
        :param time: (optional) Risk factor occurrence time (ISO8601 formatted)
        :type time: str
        """
        self.__add_evidence(self.risk_factors, _id, state, time)

    def add_evidence(self, _id, state, time=None):
        """
        Adds evidence with given presence to evidence list.

        :param _id: Evidence id
        :type _id: str
        :param state: Evidence presence,
        one of three values ("present", "absent" or "unknown")
        :type state: str
        :param time: (optional) Evidence occurrence time (ISO8601 formatted)
        :type time: str
        """
        evidence_list = self.symptoms

        if _id.startswith(("p_", "rf_")):
            evidence_list = self.risk_factors
        elif _id.startswith("lt_"):
            evidence_list = self.lab_tests

        self.__add_evidence(evidence_list, _id, state, time)

    def set_pursued_conditions(self, pursued):
        """
        Sets conditions for pursued diagnosis.

        :param pursued: List of condition ids
        :type pursued: list of strings
        """
        self.pursued = pursued

    def set_case_id(self, value):
        """
        Sets case id for diagnosis.

        :param value: Unique case id for diagnosis
        :type value: str
        """
        self.case_id = value

    def set_extras(self, attribute, value, permanent=False):
        """
        Sets extras attributes to be sent with the diagnosis requests.

        :param attribute: String with the attribute name
        :type attribute: str
        :param value: Value to set for the attribute
        :type value: bool | str | number
        :param permanent: Conditions if the attribute shall be sent only in the next diagnosis request
        or should persists through multiple diagnosis calls.
        :type permanent: bool
        """
        if permanent:
            self.extras_permanent[attribute] = value
        else:
            self.extras[attribute] = value

    def update_from_api(self, json):
        """
        Updates current object by diagnosis response from the API.

        :param json: Dict obtained from the API diagnosis response
        :type json: dict
        """
        if 'question' in json and isinstance(json['question'], dict):
            self.question = DiagnosisQuestion.from_json(json['question'])
        else:
            self.question = None

        self.conditions = ConditionResultList.from_json(json.get('conditions', []) or [])
        self.extras = json.get('extras', {}) or {}

    def get_evidences(self):
        return self.symptoms + self.lab_tests + self.risk_factors

    def get_api_request(self):
        """
        Based on current Diagnosis object construct
        dict object of the format accepted by diagnosis API method.

        :return: Diagnosis API request dict
        :rtype: dict
        """
        request = {
            "sex": self.patient_sex,
            "age": self.patient_age,

            "evidence": self.get_evidences(),

            "extras": dict(self.extras_permanent, **self.extras)
        }

        if self.pursued:
            request['pursued'] = self.pursued

        if self.evaluation_time:
            request['evaluated_at'] = self.evaluation_time

        return request

    def get_explain_request(self, target_id):
        """
        Based on current Diagnosis object construct
        dict object of the format accepted by explain API method.

        :param target_id: Condition id for which explain shall be calculated.
        :type target_id: str

        :return: Diagnosis API request dict
        :rtype: dict
        """
        request = self.get_api_request()
        request['target'] = target_id

        return request

    def to_dict(self):
        """
        Transform object to dict.

        :return: Diagnosis object as dict.
        :rtype: dict
        """
        return dict(self.get_api_request(), **{
            "question": self.question.to_dict() if hasattr(self.question, 'to_dict') else None,
            "conditions": self.conditions.to_dict() if hasattr(self.question, 'to_dict') else None
        })



# -*- coding: utf-8 -*-

"""
infermedica_api.models.risk_factor
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains risk factor models for data returned from api.
"""
from .base import BaseModel, BaseModelList


class RiskFactor(BaseModel):
    """Model class for API risk factor details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs RiskFactor object from given dict and returns it.

        :param json: Dict with risk factor values
        :type json: dict

        :returns: Risk factor details object
        :rtype: :class:`infermedica_api.models.RiskFactor`
        """
        return RiskFactor(**json)


class RiskFactorList(BaseModelList):
    """Model class for API list of risk factor details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs RiskFactorList object from list of dicts and returns it.

        :param json: List with risk factor details dict values
        :type json: list

        :returns: Risk factor details list object
        :rtype: :class:`infermedica_api.models.RiskFactorList`
        """
        mapping = {}
        for i, item in enumerate(json):
            item = RiskFactor(**item)
            mapping[item.id] = i
        return RiskFactorList(json, mapping=mapping)

    def get_risk_factor_details(self, _id):
        return self._get_details(_id)



# -*- coding: utf-8 -*-

"""
infermedica_api.models.base
~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains common models for data returned from api.
"""
import json


class ModelCommon(object):
    """Abstract class with implementation of commonly used functions."""

    class Meta:
        abstract = True

    def __str__(self):
        """Returns nicely formatted object."""
        return self.to_json(pretty_print=True)

    def to_json(self, pretty_print=False):
        """
        Returns json dumps of the class object.
        If pretty_print argument set to True,
        returned string is nicely formatted.

        :param pretty_print: Boolean which determinants
        if output should be humanly readable formatted
        :type pretty_print: bool

        :returns: String with json data of the object
        :rtype: str
        """
        if pretty_print:
            return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)
        return json.dumps(self, default=lambda o: o.__dict__)

    def to_dict(self):
        """
        Transform class object to dict.

        :return: Class object as dict.
        :rtype: dict
        """
        return dict({key: val.to_dict() if hasattr(val, 'to_dict') else val for key, val in self.__dict__.items()})


class BaseModel(ModelCommon):
    """
    Abstract class of model with init function,
    which generically assign object parameters.
    """

    class Meta:
        abstract = True

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)


class BaseModelList(list, ModelCommon):
    """
    Abstract class of list model with init function, which assign key mapping.
    """

    class Meta:
        abstract = True

    def __init__(self, *args, **kwargs):
        if 'mapping' in kwargs:
            self.mapping = kwargs.pop('mapping')
        else:
            self.mapping = {}

        super(BaseModelList, self).__init__(*args, **kwargs)

    def _get_details(self, _id):
        """
        Generic function to handle object returns by the object id.

        :param _id: Object id
        :type _id: str

        :returns: Model object with given id
        """
        try:
            return self[self.mapping[_id]]
        except (IndexError, KeyError) as e:
            return None

    def to_list(self):
        """
        Transform class object to simple list.

        :return: Class object as simple list.
        :rtype: list
        """
        return [val.to_dict() if hasattr(val, 'to_dict') else val for val in self]

    def to_dict(self):
        """
        Transform class object to list.
        It does not return dict as the name suggest, but it's created for consistency reasons.

        :return: Class object as list.
        :rtype: list
        """
        return self.to_list()



# -*- coding: utf-8 -*-

"""
infermedica_api.models.condition
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains condition models for data returned from api.
"""
from .base import BaseModel, BaseModelList


class Condition(BaseModel):
    """Model class for API condition details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs Condition object from given dict and returns it.

        :param json: Dict with condition values
        :type json: dict

        :returns: Condition details object
        :rtype: :class:`infermedica_api.models.Condition`
        """
        return Condition(**json)


class ConditionList(BaseModelList):
    """Model class for API list of condition details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs ConditionList object from list of dicts and returns it.

        :param json: List with condition details dict values
        :type json: list

        :returns: Conditions details list object
        :rtype: :class:`infermedica_api.models.ConditionList`
        """
        mapping = {}
        for i, item in enumerate(json):
            item = Condition(**item)
            mapping[item.id] = i
        return ConditionList(json, mapping=mapping)

    def get_condition_details(self, _id):
        return self._get_details(_id)



# -*- coding: utf-8 -*-

"""
infermedica_api.models.lab_test
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains laboratory test models for data returned from api.
"""
from .base import BaseModel, BaseModelList


class LabTest(BaseModel):
    """Model class for API laboratory testss details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs LabTest object from given dict and returns it.

        :param json: Dict with laboratory test values
        :type json: dict

        :returns: LabTest details object
        :rtype: :class:`infermedica_api.models.LabTest`
        """
        return LabTest(**json)


class LabTestList(BaseModelList):
    """Model class for API list of laboratory test details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs LabTestList object from list of dicts and returns it.

        :param json: List with laboratory test details dict values
        :type json: list

        :returns: LabTests details list object
        :rtype: :class:`infermedica_api.models.LabTestList`
        """
        mapping = {}
        for i, item in enumerate(json):
            item = LabTest(**item)
            mapping[item.id] = i
        return LabTestList(json, mapping=mapping)

    def get_lab_test_details(self, _id):
        return self._get_details(_id)



# -*- coding: utf-8 -*-

"""
Infermedica Python API client
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Infermedica Python API client provides access to powerful medical diagnostic API created by Infermedica.
Basic usage:

   >>> import infermedica_api
   >>> api = infermedica_api.API(app_id='YOUR_APP_ID', app_key='YOUR_APP_KEY')
   >>> print(api.info())
   {
     "updated_at": "2015-03-12T00:39:34+01:00",
     "conditions_count": 385,
     "observations_count": 875
   }

... you can also configure API object globally and use it without initialization in any module.
First configure the API:

   >>> import infermedica_api
   >>> infermedica_api.configure(app_id='YOUR_APP_ID', app_key='YOUR_APP_KEY')

... and then in any module just import infermedica module and get configured API object:

   >>> import infermedica_api
   >>> api = infermedica_api.get_api()

The other requests are presented in `examples` and are described in our documentation. Full documentation
is available at <https://developer.infermedica.com>.

:copyright: (c) 2016 by Infermedica.
:license: Apache 2.0, see LICENSE for more details.

"""

__title__ = 'Infermedica API'
__version__ = '0.0.6'
__author__ = 'Arkadiusz Szydelko'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2016 Infermedica'

DEFAULT_API_VERSION = 'v2'

API_CONFIG = {
    'v1': {
        'endpoint': 'https://api.infermedica.com/',
        'methods': {
            'info': '/info',
            'lookup': '/lookup',
            'diagnosis': '/diagnosis',
            'observations': '/observations',
            'observation_details': '/observations/{id}',
            'conditions': '/conditions',
            'condition_details': '/conditions/{id}',
            'risk_factors': '/risk_factors',
            'risk_factor_details': '/risk_factors/{id}'
        }
    },
    'v2': {
        'endpoint': 'https://api.infermedica.com/',
        'methods': {
            'info': '/info',
            'search': '/search',
            'lookup': '/lookup',
            'diagnosis': '/diagnosis',
            'explain': '/explain',
            'conditions': '/conditions',
            'condition_details': '/conditions/{id}',
            'symptoms': '/symptoms',
            'symptom_details': '/symptoms/{id}',
            'lab_tests': '/lab_tests',
            'lab_test_details': '/lab_tests/{id}',
            'risk_factors': '/risk_factors',
            'risk_factor_details': '/risk_factors/{id}'
        }
    }
}

from .models.condition import Condition, ConditionList
from .models.diagnosis import Diagnosis, DiagnosisQuestion, ConditionResult, ConditionResultList
from .models.lab_test import LabTest, LabTestList
from .models.observation import Observation, ObservationList
from .models.risk_factor import RiskFactor, RiskFactorList
from .models.symptom import Symptom, SymptomList
from .webservice import configure, get_api, API, SEARCH_FILTERS



#!/usr/bin/env python
import re

from setuptools import setup, find_packages


VERSION = ''
with open('infermedica_api/__init__.py', 'r') as f:
    VERSION = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        f.read(), re.MULTILINE).group(1)

long_description = """
    The Infermedica Python client provides access to powerful medical diagnostic API created by Infermedica.

    README and Samples - https://github.com/infermedica/python-api

    Developer portal - https://developer.infermedica.com

    API Reference - https://developer.infermedica.com/docs/api
    """

setup(
    name='infermedica-api',
    version=VERSION,
    description='The Infermedica Python client for Infermedica API.',
    long_description=long_description,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Healthcare Industry',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
    ],
    keywords='infermedica medical api library rest http',
    author='Infermedica',
    author_email='office@infermedica.com',
    url='https://github.com/infermedica/python-api',
    license='Apache 2.0',
    packages=find_packages(exclude=['examples']),
    install_requires=['requests>=1.0.0'],
)



from .base import BaseModel


class ExplainResults(BaseModel):
    """Model class for API explain object."""

    def __init__(self, **kwargs):
        super(ExplainResults, self).__init__(**kwargs)

        self.supporting_evidence = [ExplainResult.from_json(val) for val in self.supporting_evidence]
        self.conflicting_evidence = [ExplainResult.from_json(val) for val in self.conflicting_evidence]

    @staticmethod
    def from_json(json):
        """
        Constructs ExplainResults object from given dict and returns it.

        :param json: Dict with explain results value.
        :type json: dict

        :returns: Explain result details object
        :rtype: :class:`infermedica_api.models.ExplainResults`
        """
        return ExplainResults(**json)


class ExplainResult(BaseModel):
    """Model class for handling single explain result object."""

    @staticmethod
    def from_json(json):
        """
        Constructs ExplainResult object from given dict and returns it.

        :param json: Dict with single explain result value.
        :type json: dict

        :returns: Explain result details object
        :rtype: :class:`infermedica_api.models.ExplainResult`
        """
        return ExplainResult(**json)



from .condition import Condition, ConditionList
from .diagnosis import Diagnosis, DiagnosisQuestion, ConditionResult, ConditionResultList
from .lab_test import LabTest, LabTestList
from .observation import Observation, ObservationList
from .risk_factor import RiskFactor, RiskFactorList
from .symptom import Symptom, SymptomList
from .explain import ExplainResults, ExplainResult



# -*- coding: utf-8 -*-

"""
infermedica_api.models.observation
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains observation models for data returned from api.
"""
import warnings

from .base import BaseModel, BaseModelList


class Observation(BaseModel):
    """Model class for API observations details objects."""
    # TODO: Children ids
    @staticmethod
    def from_json(json):
        """
        Constructs Observation object from given dict and returns it.

        :param json: Dict with observation values
        :type json: dict

        :returns: Observation details object
        :rtype: :class:`infermedica_api.models.Observation`
        """
        warnings.warn("Class Observation is deprecated, please use Symptom.",
                      category=DeprecationWarning)
        return Observation(**json)


class ObservationList(BaseModelList):
    """Model class for API list of observation details objects."""

    @staticmethod
    def from_json(json):
        """
        Constructs ObservationList object from list of dicts and returns it.

        :param json: List with observation details dict values
        :type json: list

        :returns: Observations details list object
        :rtype: :class:`infermedica_api.models.ObservationList`
        """
        warnings.warn("Class ObservationList is deprecated, please use SymptomList.",
                      category=DeprecationWarning)
        mapping = {}
        for i, item in enumerate(json):
            item = Observation(**item)
            mapping[item.id] = i
        return ObservationList(json, mapping=mapping)

    def get_observation_details(self, _id):
        return self._get_details(_id)
